package com.hzya.frame.u9c.saledeliver.dao.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.hzya.frame.u9c.saledeliver.entity.VU9cSaledeliverEntity;
import com.hzya.frame.u9c.saledeliver.dao.IVU9cSaledeliverDao;
import org.springframework.stereotype.Repository;
import com.hzya.frame.basedao.dao.MybatisGenericDao;

import java.util.List;

/**
 * (VU9cSaledeliver)表数据库访问层
 *
 * @author makejava
 * @since 2023-10-26 11:29:50
 */
@Repository("VU9cSaledeliverDaoImpl")
public class VU9cSaledeliverDaoImpl extends MybatisGenericDao<VU9cSaledeliverEntity, String> implements IVU9cSaledeliverDao{
    @DS("henghua_sqlserver_test_xel")
    @Override
    public List<VU9cSaledeliverEntity> getu9cSaledeliver(VU9cSaledeliverEntity category) {
        return (List<VU9cSaledeliverEntity>) super.selectList("com.hzya.frame.u9c.saledeliver.dao.impl.VU9cSaledeliverEntityDaoImpl.VU9cSaledeliverEntity_list_base",category);
    }
    @DS("henghua_sqlserver_test_xel")
    @Override
    public int updateCrmId(VU9cSaledeliverEntity itemmaster) {
        return super.update("com.hzya.frame.u9c.saledeliver.dao.impl.VU9cSaledeliverEntityDaoImpl.entity_update",itemmaster);
    }
    @DS("henghua_sqlserver_test_xel")
    @Override
    public List<VU9cSaledeliverEntity> getu9cSaledeliverForDelete(VU9cSaledeliverEntity category) {
        return (List<VU9cSaledeliverEntity>) super.selectList("com.hzya.frame.u9c.saledeliver.dao.impl.VU9cSaledeliverEntityDaoImpl.VU9cSaledeliverEntity_list_base_for_delete",category);
    }
    @DS("henghua_sqlserver_test_xel")
    @Override
    public int updateDeleteState(VU9cSaledeliverEntity entity) {
        return super.update("com.hzya.frame.u9c.saledeliver.dao.impl.VU9cSaledeliverEntityDaoImpl.entity_update_for_delete",entity);
    }
    @DS("henghua_sqlserver_test_xel")
    @Override
    public List<VU9cSaledeliverEntity> getu9cSaledeliverForUpdate(VU9cSaledeliverEntity category) {
        return (List<VU9cSaledeliverEntity>) super.selectList("com.hzya.frame.u9c.saledeliver.dao.impl.VU9cSaledeliverEntityDaoImpl.VU9cSaledeliverEntity_list_base_for_update",category);
    }
    @DS("henghua_sqlserver_test_xel")
    @Override
    public List<VU9cSaledeliverEntity> getu9cSaledeliverForUpdateDetails(VU9cSaledeliverEntity category) {
        return (List<VU9cSaledeliverEntity>) super.selectList("com.hzya.frame.u9c.saledeliver.dao.impl.VU9cSaledeliverEntityDaoImpl.VU9cSaledeliverEntity_list_base_for_update_details",category);
    }
}

